@@ -32,10 +32,6 @@
 #include "gis-assistant-private.h"
 #include "gis-center-container.h"
 
-G_DEFINE_TYPE (GisAssistant, gis_assistant, GTK_TYPE_BOX)
-
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GIS_TYPE_ASSISTANT, GisAssistantPrivate))
-
 enum {
   PROP_0,
   PROP_TITLE,
@@ -45,7 +41,6 @@ enum {
 static GParamSpec *obj_props[PROP_LAST];
 
 enum {
-  PREPARE,
   NEXT_PAGE,
   LAST_SIGNAL,
 };
@@ -63,10 +58,14 @@ struct _GisAssistantPrivate
   GtkWidget *action_area;
   GtkWidget *page_action_widget_area;
   GtkWidget *spinner;
+  GtkWidget *titlebar;
 
   GList *pages;
   GisPage *current_page;
 };
+typedef struct _GisAssistantPrivate GisAssistantPrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GisAssistant, gis_assistant, GTK_TYPE_BOX)
 
 struct _GisAssistantPagePrivate
 {
@@ -78,7 +77,7 @@ widget_destroyed (GtkWidget    *widget,
                   GisAssistant *assistant)
 {
   GisPage *page = GIS_PAGE (widget);
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
 
   priv->pages = g_list_delete_link (priv->pages, page->assistant_priv->link);
   if (page == priv->current_page)
@@ -102,16 +101,19 @@ on_apply_done (GisPage *page,
                gpointer user_data)
 {
   GisAssistant *assistant = GIS_ASSISTANT (user_data);
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+
   if (valid)
     g_signal_emit (assistant, signals[NEXT_PAGE], 0,
-                   assistant->priv->current_page);
+                   priv->current_page);
+
   g_object_unref (assistant);
 }
 
 void
 gis_assistant_next_page (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   if (priv->current_page)
     gis_page_apply_begin (priv->current_page, on_apply_done,
                           g_object_ref (assistant));
@@ -156,7 +158,7 @@ find_prev_page (GisPage *page)
 void
 gis_assistant_previous_page (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   g_return_if_fail (priv->current_page != NULL);
   gis_assistant_switch_to (assistant, GIS_ASSISTANT_PREV, find_prev_page (priv->current_page));
 }
@@ -172,7 +174,7 @@ remove_from_page_action_area (GtkWidget *widget,
 static void
 update_action_widget (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   GtkWidget *action;
 
   gtk_container_foreach (GTK_CONTAINER (priv->page_action_widget_area),
@@ -195,7 +197,7 @@ remove_from_progress_indicator (GtkWidget *widget,
 static void
 update_progress_indicator (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   GList *l;
 
   gtk_container_foreach (GTK_CONTAINER (priv->progress_indicator),
@@ -220,7 +222,7 @@ update_progress_indicator (GisAssistant *assistant)
 static void
 update_navigation_buttons (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   GisPage *page = priv->current_page;
   GisAssistantPagePrivate *page_priv;
   gboolean is_last_page;
@@ -249,6 +251,10 @@ update_navigation_buttons (GisAssistant *assistant)
 
       can_go_forward = gis_page_get_complete (page);
       gtk_widget_set_sensitive (priv->forward, can_go_forward);
+      if (can_go_forward)
+        gtk_style_context_add_class (gtk_widget_get_style_context (priv->forward), "suggested-action");
+      else
+        gtk_style_context_remove_class (gtk_widget_get_style_context (priv->forward), "suggested-action");
     }
 }
 
@@ -256,27 +262,28 @@ static void
 update_applying_state (GisAssistant *assistant)
 {
   gboolean applying = FALSE;
-  if (assistant->priv->current_page)
-    applying = gis_page_get_applying (assistant->priv->current_page);
-  gtk_widget_set_sensitive (assistant->priv->frame, !applying);
-  gtk_widget_set_sensitive (assistant->priv->forward, !applying);
-  gtk_widget_set_visible (assistant->priv->back, !applying);
-  gtk_widget_set_visible (assistant->priv->cancel, applying);
-  gtk_widget_set_visible (assistant->priv->spinner, applying);
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  if (priv->current_page)
+    applying = gis_page_get_applying (priv->current_page);
+  gtk_widget_set_sensitive (priv->frame, !applying);
+  gtk_widget_set_sensitive (priv->forward, !applying);
+  gtk_widget_set_visible (priv->back, !applying);
+  gtk_widget_set_visible (priv->cancel, applying);
+  gtk_widget_set_visible (priv->spinner, applying);
 
   if (applying)
-    gtk_spinner_start (GTK_SPINNER (assistant->priv->spinner));
+    gtk_spinner_start (GTK_SPINNER (priv->spinner));
   else
-    gtk_spinner_stop (GTK_SPINNER (assistant->priv->spinner));
+    gtk_spinner_stop (GTK_SPINNER (priv->spinner));
 }
 
 static void
-gis_assistant_real_prepare (GisAssistant *assistant,
-                            GisPage      *page)
+update_titlebar (GisAssistant *assistant)
 {
-  update_action_widget (assistant);
-  update_navigation_buttons (assistant);
-  update_progress_indicator (assistant);
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+
+  gtk_header_bar_set_title (GTK_HEADER_BAR (priv->titlebar),
+                            gis_assistant_get_title (assistant));
 }
 
 static void
@@ -284,7 +291,9 @@ page_notify (GisPage      *page,
              GParamSpec   *pspec,
              GisAssistant *assistant)
 {
-  if (page != assistant->priv->current_page)
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+
+  if (page != priv->current_page)
     return;
 
   if (strcmp (pspec->name, "title") == 0)
@@ -299,7 +308,7 @@ void
 gis_assistant_add_page (GisAssistant *assistant,
                         GisPage      *page)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   GList *link;
 
   g_return_if_fail (page->assistant_priv == NULL);
@@ -322,14 +331,14 @@ gis_assistant_add_page (GisAssistant *assistant,
 GisPage *
 gis_assistant_get_current_page (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   return priv->current_page;
 }
 
 GList *
 gis_assistant_get_all_pages (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   return priv->pages;
 }
 
@@ -351,44 +360,59 @@ static void
 do_cancel (GtkWidget    *button,
            GisAssistant *assistant)
 {
-  if (assistant->priv->current_page)
-    gis_page_apply_cancel (assistant->priv->current_page);
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  if (priv->current_page)
+    gis_page_apply_cancel (priv->current_page);
 }
 
-gchar *
+const gchar *
 gis_assistant_get_title (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   if (priv->current_page != NULL)
     return gis_page_get_title (priv->current_page);
   else
     return "";
 }
 
+GtkWidget *
+gis_assistant_get_titlebar (GisAssistant *assistant)
+{
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  return priv->titlebar;
+}
+
 GtkWidget *
 _gis_assistant_get_frame (GisAssistant *assistant)
 {
-  return assistant->priv->frame;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  return priv->frame;
 }
 
 void
 _gis_assistant_current_page_changed (GisAssistant *assistant,
                                      GisPage      *page)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
 
-  if (priv->current_page != page) {
-    priv->current_page = page;
-    update_applying_state (assistant);
-    g_object_notify_by_pspec (G_OBJECT (assistant), obj_props[PROP_TITLE]);
-    g_signal_emit (assistant, signals[PREPARE], 0, page);
-  }
+  if (priv->current_page == page)
+    return;
+
+  priv->current_page = page;
+  g_object_notify_by_pspec (G_OBJECT (assistant), obj_props[PROP_TITLE]);
+
+  update_titlebar (assistant);
+  update_applying_state (assistant);
+  update_action_widget (assistant);
+  update_navigation_buttons (assistant);
+  update_progress_indicator (assistant);
+  gis_page_shown (page);
 }
 
 void
 gis_assistant_locale_changed (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = assistant->priv;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   GList *l;
 
   gtk_button_set_label (GTK_BUTTON (priv->forward), _("_Next"));
@@ -399,14 +423,24 @@ gis_assistant_locale_changed (GisAssistant *assistant)
     gis_page_locale_changed (l->data);
 }
 
+void
+gis_assistant_save_data (GisAssistant *assistant)
+{
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  GList *l;
+
+  for (l = priv->pages; l != NULL; l = l->next)
+    gis_page_save_data (l->data);
+}
+
 static void
 gis_assistant_init (GisAssistant *assistant)
 {
-  GisAssistantPrivate *priv = GET_PRIVATE (assistant);
-  GtkWidget *navigation;
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   GtkWidget *widget;
 
-  assistant->priv = priv;
+  priv->titlebar = gtk_header_bar_new ();
+  gtk_widget_show (priv->titlebar);
 
   priv->main_layout = gtk_box_new (GTK_ORIENTATION_VERTICAL, 20);
   gtk_box_pack_start (GTK_BOX (assistant), priv->main_layout, TRUE, TRUE, 0);
@@ -417,11 +451,6 @@ gis_assistant_init (GisAssistant *assistant)
 
   priv->page_action_widget_area = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
-  navigation = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_box_set_homogeneous (GTK_BOX (navigation), TRUE);
-  gtk_widget_set_halign (navigation, GTK_ALIGN_END);
-  gtk_widget_set_hexpand (navigation, TRUE);
-
   priv->forward = gtk_button_new ();
   gtk_button_set_image (GTK_BUTTON (priv->forward),
                         gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON));
@@ -438,16 +467,17 @@ gis_assistant_init (GisAssistant *assistant)
                         gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
   gtk_button_set_use_underline (GTK_BUTTON (priv->cancel), TRUE);
 
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->titlebar), priv->cancel);
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->titlebar), priv->back);
+
   priv->spinner = gtk_spinner_new ();
   widget = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (widget), 0, 0, 12, 6);
-  gtk_box_pack_start (GTK_BOX (navigation), widget, FALSE, FALSE, 0);
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->titlebar), widget);
   gtk_container_add (GTK_CONTAINER (widget), priv->spinner);
   gtk_widget_show (widget);
 
-  gtk_box_pack_start (GTK_BOX (navigation), priv->cancel, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (navigation), priv->back, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (navigation), priv->forward, FALSE, FALSE, 0);
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->titlebar), priv->forward);
 
   g_signal_connect (priv->forward, "clicked", G_CALLBACK (go_forward), assistant);
   g_signal_connect (priv->back, "clicked", G_CALLBACK (go_backward), assistant);
@@ -457,8 +487,7 @@ gis_assistant_init (GisAssistant *assistant)
   gtk_widget_set_halign (priv->progress_indicator, GTK_ALIGN_CENTER);
 
   priv->action_area = gis_center_container_new (priv->page_action_widget_area,
-                                                priv->progress_indicator,
-                                                navigation);
+                                                priv->progress_indicator);
 
   gis_assistant_locale_changed (assistant);
   gtk_box_pack_start (GTK_BOX (priv->main_layout), priv->action_area, FALSE, TRUE, 0);
@@ -492,11 +521,8 @@ gis_assistant_class_init (GisAssistantClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GisAssistantPrivate));
-
   gobject_class->get_property = gis_assistant_get_property;
 
-  klass->prepare = gis_assistant_real_prepare;
   klass->next_page = gis_assistant_real_next_page;
 
   obj_props[PROP_TITLE] =
@@ -522,24 +548,4 @@ gis_assistant_class_init (GisAssistantClass *klass)
                   G_STRUCT_OFFSET (GisAssistantClass, next_page),
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 1, GIS_TYPE_PAGE);
-
-  /**
-   * GisAssistant::prepare:
-   * @assistant: the #GisAssistant
-   * @page: the current page
-   *
-   * The ::prepare signal is emitted when a new page is set as the
-   * assistant's current page, before making the new page visible.
-   *
-   * A handler for this signal can do any preparations which are
-   * necessary before showing @page.
-   */
-  signals[PREPARE] =
-    g_signal_new ("prepare",
-                  G_TYPE_FROM_CLASS (gobject_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (GisAssistantClass, prepare),
-                  NULL, NULL, NULL,
-                  G_TYPE_NONE, 1, GIS_TYPE_PAGE);
-
 }
