@@ -53,10 +53,6 @@ static void        on_realm_joined     (GObject *source,
                                         GAsyncResult *result,
                                         gpointer user_data);
 
-G_DEFINE_TYPE (GisAccountPage, gis_account_page, GIS_TYPE_PAGE);
-
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GIS_TYPE_ACCOUNT_PAGE, GisAccountPagePrivate))
-
 typedef enum {
   UM_LOCAL,
   UM_ENTERPRISE,
@@ -77,8 +73,6 @@ struct _GisAccountPagePrivate
   guint reason_timeout;
   ActUserAccountType account_type;
 
-  gboolean user_data_unsaved;
-
   gboolean has_enterprise;
   guint realmd_watch;
   UmRealmManager *realm_manager;
@@ -91,6 +85,9 @@ struct _GisAccountPagePrivate
 
   GtkWidget *action;
 };
+typedef struct _GisAccountPagePrivate GisAccountPagePrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GisAccountPage, gis_account_page, GIS_TYPE_PAGE);
 
 #define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE (page)->builder,(name)))
 #define WID(name) OBJ(GtkWidget*,name)
@@ -124,7 +121,7 @@ show_error_dialog (GisAccountPage *page,
 static void
 clear_account_page (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkWidget *fullname_entry;
   GtkWidget *username_combo;
   GtkWidget *password_entry;
@@ -142,8 +139,6 @@ clear_account_page (GisAccountPage *page)
   /* FIXME: change this for a large deployment scenario; maybe through a GSetting? */
   priv->account_type = ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR;
 
-  priv->user_data_unsaved = FALSE;
-
   gtk_entry_set_text (GTK_ENTRY (fullname_entry), "");
   gtk_list_store_clear (GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (username_combo))));
   gtk_entry_set_text (GTK_ENTRY (password_entry), "");
@@ -153,7 +148,7 @@ clear_account_page (GisAccountPage *page)
 static gboolean
 local_validate (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
 
   return priv->valid_name &&
          priv->valid_username &&
@@ -186,7 +181,8 @@ enterprise_validate (GisAccountPage *page)
 static gboolean
 page_validate (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
+
   switch (priv->mode) {
   case UM_LOCAL:
     return local_validate (page);
@@ -207,7 +203,7 @@ static void
 set_mode (GisAccountPage *page,
           UmAccountMode   mode)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkWidget *nb;
 
   if (priv->mode == mode)
@@ -225,7 +221,7 @@ static void
 set_has_enterprise (GisAccountPage *page,
                     gboolean        has_enterprise)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
 
   if (priv->has_enterprise == has_enterprise)
     return;
@@ -241,7 +237,7 @@ set_has_enterprise (GisAccountPage *page,
 static void
 update_valid_confirm (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   const gchar *password, *verify;
 
   password = gtk_entry_get_text (GTK_ENTRY (WID("account-password-entry")));
@@ -255,7 +251,7 @@ fullname_changed (GtkWidget      *w,
                   GParamSpec     *pspec,
                   GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkWidget *combo;
   GtkWidget *entry;
   GtkTreeModel *model;
@@ -270,7 +266,6 @@ fullname_changed (GtkWidget      *w,
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
   priv->valid_name = is_valid_name (name);
-  priv->user_data_unsaved = TRUE;
 
   if (!priv->valid_name) {
     gtk_entry_set_text (GTK_ENTRY (entry), "");
@@ -288,7 +283,7 @@ static void
 username_changed (GtkComboBoxText *combo,
                   GisAccountPage  *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   const gchar *username;
   gchar *tip;
   GtkWidget *entry;
@@ -296,7 +291,6 @@ username_changed (GtkComboBoxText *combo,
   username = gtk_combo_box_text_get_active_text (combo);
 
   priv->valid_username = is_valid_username (username, &tip);
-  priv->user_data_unsaved = TRUE;
 
   entry = gtk_bin_get_child (GTK_BIN (combo));
 
@@ -319,7 +313,7 @@ static gboolean
 reason_timeout_cb (gpointer data)
 {
   GisAccountPage *page = GIS_ACCOUNT_PAGE (data);
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkWidget *password_entry;
   GtkWidget *confirm_entry;
   const gchar *password;
@@ -346,7 +340,7 @@ reason_timeout_cb (gpointer data)
 static void
 refresh_reason_timeout (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
 
   if (priv->reason_timeout != 0)
     g_source_remove (priv->reason_timeout);
@@ -357,7 +351,7 @@ refresh_reason_timeout (GisAccountPage *page)
 static void
 update_password_entries (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   const gchar *password;
   const gchar *username;
   GtkWidget *password_entry;
@@ -409,10 +403,8 @@ password_changed (GtkWidget      *w,
                   GParamSpec     *pspec,
                   GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
   clear_entry_validation_error (GTK_ENTRY (w));
   update_password_entries (page);
-  priv->user_data_unsaved = TRUE;
   update_account_page_status (page);
 }
 
@@ -421,8 +413,10 @@ password_entry_focus_out (GtkWidget      *widget,
                           GdkEventFocus  *event,
                           GisAccountPage *page)
 {
-  if (page->priv->reason_timeout != 0)
-    g_source_remove (page->priv->reason_timeout);
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
+
+  if (priv->reason_timeout != 0)
+    g_source_remove (priv->reason_timeout);
 
   return FALSE;
 }
@@ -432,7 +426,7 @@ confirm_entry_focus_out (GtkWidget      *widget,
                          GdkEventFocus  *event,
                          GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkEntry *entry = GTK_ENTRY (widget);
   const gchar *verify;
 
@@ -447,90 +441,26 @@ confirm_entry_focus_out (GtkWidget      *widget,
 }
 
 static void
-create_user (GisAccountPage *page)
+local_create_user (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   const gchar *username;
+  const gchar *password;
   const gchar *fullname;
   const gchar *language;
-  GError *error;
+  GError *error = NULL;
 
   username = gtk_combo_box_text_get_active_text (OBJ(GtkComboBoxText*, "account-username-combo"));
-  fullname = gtk_entry_get_text (OBJ(GtkEntry*, "account-fullname-entry"));
-
-  error = NULL;
+  fullname = gtk_entry_get_text (OBJ (GtkEntry*, "account-fullname-entry"));
+  password = gtk_entry_get_text (OBJ (GtkEntry*, "account-password-entry"));
 
   priv->act_user = act_user_manager_create_user (priv->act_client, username, fullname, priv->account_type, &error);
-  language = gis_driver_get_user_language (GIS_PAGE (page)->driver);
-  if (language)
-    act_user_set_language (priv->act_user, language);
-
   if (error != NULL) {
     g_warning ("Failed to create user: %s", error->message);
     g_error_free (error);
-  }
-  /* TODO:We don't support coming back to this page to modify the values after
-   * the user has been created. For now just disable it
-   */
-  gtk_widget_set_sensitive (GTK_WIDGET (page), FALSE);
-  gtk_widget_set_sensitive (priv->action, FALSE);
-}
-
-static void save_account_data (GisAccountPage *page);
-
-static gulong when_loaded;
-
-static void
-save_when_loaded (ActUser        *user,
-                  GParamSpec     *pspec,
-                  GisAccountPage *page)
-{
-  g_signal_handler_disconnect (user, when_loaded);
-  when_loaded = 0;
-
-  save_account_data (page);
-}
-
-static void
-local_create_user (GisAccountPage *page)
-{
-  GisAccountPagePrivate *priv = page->priv;
-  const gchar *realname;
-  const gchar *username;
-  const gchar *password;
-
-  if (!priv->user_data_unsaved) {
     return;
   }
 
-  /* this can happen when going back */
-  if (!priv->valid_name ||
-      !priv->valid_username) {
-    return;
-  }
-
-  if (priv->act_user == NULL) {
-    create_user (page);
-  }
-
-  if (priv->act_user == NULL) {
-    g_warning ("User creation failed");
-    clear_account_page (page);
-    return;
-  }
-
-  if (!act_user_is_loaded (priv->act_user)) {
-    if (when_loaded == 0)
-      when_loaded = g_signal_connect (priv->act_user, "notify::is-loaded",
-                                      G_CALLBACK (save_when_loaded), page);
-    return;
-  }
-
-  realname = gtk_entry_get_text (OBJ (GtkEntry*, "account-fullname-entry"));
-  username = gtk_combo_box_text_get_active_text (OBJ (GtkComboBoxText*, "account-username-combo"));
-  password = gtk_entry_get_text (OBJ (GtkEntry*, "account-password-entry"));
-
-  act_user_set_real_name (priv->act_user, realname);
   act_user_set_user_name (priv->act_user, username);
   act_user_set_account_type (priv->act_user, priv->account_type);
   if (strlen (password) == 0)
@@ -538,11 +468,13 @@ local_create_user (GisAccountPage *page)
   else
     act_user_set_password (priv->act_user, password, "");
 
+  language = gis_driver_get_user_language (GIS_PAGE (page)->driver);
+  if (language)
+    act_user_set_language (priv->act_user, language);
+
   gis_driver_set_user_permissions (GIS_PAGE (page)->driver,
                                    priv->act_user,
                                    password);
-
-  priv->user_data_unsaved = FALSE;
 }
 
 static void
@@ -551,8 +483,7 @@ on_permit_user_login (GObject *source,
                       gpointer user_data)
 {
   GisAccountPage *page = user_data;
-  GisAccountPagePrivate *priv = page->priv;
-
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   UmRealmCommon *common;
   GError *error = NULL;
   gchar *login;
@@ -586,6 +517,7 @@ on_permit_user_login (GObject *source,
 static void
 enterprise_permit_user_login (GisAccountPage *page, UmRealmObject *realm)
 {
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   UmRealmCommon *common;
   gchar *login;
   const gchar *add[2];
@@ -606,7 +538,7 @@ enterprise_permit_user_login (GisAccountPage *page, UmRealmObject *realm)
 
   um_realm_common_call_change_login_policy (common, "",
                                             add, remove, options,
-                                            page->priv->cancellable,
+                                            priv->cancellable,
                                             on_permit_user_login,
                                             page);
 
@@ -620,6 +552,7 @@ on_set_static_hostname (GObject *source,
                         gpointer user_data)
 {
   GisAccountPage *page = user_data;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GError *error = NULL;
   GVariant *retval;
   const gchar *name;
@@ -637,9 +570,9 @@ on_set_static_hostname (GObject *source,
   g_debug ("Logging in as admin user: %s", name);
 
   /* Prompted for some admin credentials, try to use them to log in */
-  um_realm_login (page->priv->realm, name,
+  um_realm_login (priv->realm, name,
                   gtk_entry_get_text (OBJ (GtkEntry *, "join-password")),
-                  page->priv->cancellable, on_join_login, page);
+                  priv->cancellable, on_join_login, page);
 }
 
 static void
@@ -648,6 +581,7 @@ on_join_response (GtkDialog *dialog,
                   gpointer user_data)
 {
   GisAccountPage *page = user_data;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GDBusConnection *connection;
   GError *error = NULL;
   gchar hostname[128];
@@ -664,7 +598,7 @@ on_join_response (GtkDialog *dialog,
       !g_str_equal (name, hostname)) {
     g_debug ("Setting StaticHostname to '%s'", name);
 
-    connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, page->priv->cancellable, &error);
+    connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, priv->cancellable, &error);
     if (error != NULL) {
       gis_page_apply_complete (GIS_PAGE (page), FALSE);
       g_warning ("Could not get DBus connection: %s", error->message);
@@ -685,7 +619,7 @@ on_join_response (GtkDialog *dialog,
     g_debug ("Logging in as admin user: %s", name);
 
     /* Prompted for some admin credentials, try to use them to log in */
-    um_realm_login (page->priv->realm, name,
+    um_realm_login (priv->realm, name,
                     gtk_entry_get_text (OBJ (GtkEntry *, "join-password")),
                     NULL, on_join_login, page);
   }
@@ -695,6 +629,7 @@ static void
 join_show_prompt (GisAccountPage *page,
                   GError *error)
 {
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   UmRealmKerberosMembership *membership;
   UmRealmKerberos *kerberos;
   gchar hostname[128];
@@ -714,8 +649,8 @@ join_show_prompt (GisAccountPage *page,
   gtk_entry_set_text (join_password, "");
   gtk_widget_grab_focus (GTK_WIDGET (join_password));
 
-  kerberos = um_realm_object_get_kerberos (page->priv->realm);
-  membership = um_realm_object_get_kerberos_membership (page->priv->realm);
+  kerberos = um_realm_object_get_kerberos (priv->realm);
+  membership = um_realm_object_get_kerberos_membership (priv->realm);
 
   gtk_label_set_text (join_domain,
                       um_realm_kerberos_get_domain_name (kerberos));
@@ -726,7 +661,7 @@ join_show_prompt (GisAccountPage *page,
   clear_entry_validation_error (join_name);
   clear_entry_validation_error (join_password);
 
-  if (!page->priv->join_prompted) {
+  if (!priv->join_prompted) {
     name = um_realm_kerberos_membership_get_suggested_administrator (membership);
     if (name && !g_str_equal (name, "")) {
       g_debug ("Suggesting admin user: %s", name);
@@ -755,7 +690,7 @@ join_show_prompt (GisAccountPage *page,
   gtk_window_set_modal (GTK_WINDOW (join_dialog), TRUE);
   gtk_window_present (GTK_WINDOW (join_dialog));
 
-  page->priv->join_prompted = TRUE;
+  priv->join_prompted = TRUE;
   g_object_unref (kerberos);
   g_object_unref (membership);
 
@@ -768,14 +703,15 @@ on_join_login (GObject *source,
                gpointer user_data)
 {
   GisAccountPage *page = user_data;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GError *error = NULL;
   GBytes *creds;
 
-  um_realm_login_finish (page->priv->realm, result, &creds, &error);
+  um_realm_login_finish (priv->realm, result, &creds, &error);
 
   /* Logged in as admin successfully, use creds to join domain */
   if (error == NULL) {
-    if (!um_realm_join_as_admin (page->priv->realm,
+    if (!um_realm_join_as_admin (priv->realm,
                                  gtk_entry_get_text (OBJ (GtkEntry *, "join-name")),
                                  gtk_entry_get_text (OBJ (GtkEntry *, "join-password")),
                                  creds, NULL, on_realm_joined, page)) {
@@ -799,14 +735,15 @@ on_realm_joined (GObject *source,
                  gpointer user_data)
 {
   GisAccountPage *page = user_data;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GError *error = NULL;
 
-  um_realm_join_finish (page->priv->realm, result, &error);
+  um_realm_join_finish (priv->realm, result, &error);
 
   /* Yay, joined the domain, register the user locally */
   if (error == NULL) {
     g_debug ("Joining realm completed successfully");
-    enterprise_permit_user_login (page, page->priv->realm);
+    enterprise_permit_user_login (page, priv->realm);
 
     /* Credential failure while joining domain, prompt for admin creds */
   } else if (g_error_matches (error, UM_REALM_ERROR, UM_REALM_ERROR_BAD_LOGIN) ||
@@ -832,10 +769,11 @@ on_realm_login (GObject *source,
                 gpointer user_data)
 {
   GisAccountPage *page = user_data;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GError *error = NULL;
   GBytes *creds = NULL;
 
-  um_realm_login_finish (page->priv->realm, result, &creds, &error);
+  um_realm_login_finish (priv->realm, result, &creds, &error);
 
   /*
    * User login is valid, but cannot authenticate right now (eg: user needs
@@ -849,16 +787,16 @@ on_realm_login (GObject *source,
   if (error == NULL) {
 
     /* Already joined to the domain, just register this user */
-    if (um_realm_is_configured (page->priv->realm)) {
+    if (um_realm_is_configured (priv->realm)) {
       g_debug ("Already joined to this realm");
-      enterprise_permit_user_login (page, page->priv->realm);
+      enterprise_permit_user_login (page, priv->realm);
 
       /* Join the domain, try using the user's creds */
     } else if (creds == NULL ||
-               !um_realm_join_as_user (page->priv->realm,
+               !um_realm_join_as_user (priv->realm,
                                        gtk_entry_get_text (OBJ (GtkEntry *, "enterprise-login")),
                                        gtk_entry_get_text (OBJ (GtkEntry *, "enterprise-password")),
-                                       creds, page->priv->cancellable,
+                                       creds, priv->cancellable,
                                        on_realm_joined,
                                        page)) {
 
@@ -896,12 +834,14 @@ on_realm_login (GObject *source,
 static void
 enterprise_check_login (GisAccountPage *page)
 {
-  g_assert (page->priv->realm);
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
 
-  um_realm_login (page->priv->realm,
+  g_assert (priv->realm);
+
+  um_realm_login (priv->realm,
                   gtk_entry_get_text (OBJ (GtkEntry *, "enterprise-login")),
                   gtk_entry_get_text (OBJ (GtkEntry *, "enterprise-password")),
-                  page->priv->cancellable,
+                  priv->cancellable,
                   on_realm_login,
                   page);
 }
@@ -912,7 +852,7 @@ on_realm_discover_input (GObject *source,
                          gpointer user_data)
 {
   GisAccountPage *page = user_data;
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GError *error = NULL;
   GList *realms;
 
@@ -940,7 +880,7 @@ on_realm_discover_input (GObject *source,
 static void
 enterprise_add_user (GisAccountPage *page)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkTreeIter iter;
   GtkComboBox *domain = OBJ(GtkComboBox*, "enterprise-domain");
 
@@ -963,23 +903,6 @@ enterprise_add_user (GisAccountPage *page)
   }
 }
 
-static void
-save_account_data (GisAccountPage *page)
-{
-  GisAccountPagePrivate *priv = page->priv;
-  switch (priv->mode) {
-  case UM_LOCAL:
-    local_create_user (page);
-    gis_page_apply_complete (GIS_PAGE (page), TRUE);
-    break;
-  case UM_ENTERPRISE:
-    enterprise_add_user (page);
-    break;
-  default:
-    g_assert_not_reached ();
-  }
-}
-
 static gchar *
 realm_get_name (UmRealmObject *realm)
 {
@@ -1023,7 +946,7 @@ static void
 enterprise_add_realm (GisAccountPage *page,
                       UmRealmObject  *realm)
 {
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkTreeIter iter;
   GtkComboBox *domain = OBJ(GtkComboBox*, "enterprise-domain");
   GtkTreeModel *model = OBJ(GtkTreeModel*, "enterprise-realms-model");
@@ -1070,7 +993,7 @@ on_realm_manager_created (GObject *source,
                           gpointer user_data)
 {
   GisAccountPage *page = user_data;
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GError *error = NULL;
   GList *realms, *l;
 
@@ -1113,7 +1036,7 @@ on_realmd_disappeared (GDBusConnection *unused1,
                        gpointer user_data)
 {
   GisAccountPage *page = user_data;
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
 
   if (priv->realm_manager != NULL) {
     g_signal_handlers_disconnect_by_func (priv->realm_manager,
@@ -1130,7 +1053,9 @@ on_domain_changed (GtkComboBox *widget,
                    gpointer user_data)
 {
   GisAccountPage *page = user_data;
-  page->priv->domain_chosen = TRUE;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
+
+  priv->domain_chosen = TRUE;
   update_account_page_status (page);
   clear_entry_validation_error (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (widget))));
 }
@@ -1152,22 +1077,47 @@ toggle_mode (GtkToggleButton *button,
             gtk_toggle_button_get_active (button) ? UM_ENTERPRISE : UM_LOCAL);
 }
 
-static void
+static gboolean
 gis_account_page_apply (GisPage *gis_page,
                         GCancellable *cancellable)
 {
   GisAccountPage *page = GIS_ACCOUNT_PAGE (gis_page);
-  g_clear_object (&page->priv->cancellable);
-  page->priv->cancellable = g_object_ref (cancellable);
-  save_account_data (page);
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
+
+  switch (priv->mode) {
+  case UM_LOCAL:
+    return FALSE;
+  case UM_ENTERPRISE:
+    priv->cancellable = g_object_ref (cancellable);
+    enterprise_add_user (page);
+    return TRUE;
+  default:
+    g_assert_not_reached ();
+  }
+}
+
+static void
+gis_account_page_save_data (GisPage *gis_page)
+{
+  GisAccountPage *page = GIS_ACCOUNT_PAGE (gis_page);
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
+
+  switch (priv->mode) {
+  case UM_LOCAL:
+    local_create_user (page);
+    break;
+  case UM_ENTERPRISE:
+    break;
+  default:
+    g_assert_not_reached ();
+  }
 }
 
 static void
 gis_account_page_constructed (GObject *object)
 {
   GisAccountPage *page = GIS_ACCOUNT_PAGE (object);
-  GisAccountPagePrivate *priv = page->priv;
-
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
   GtkWidget *fullname_entry;
   GtkWidget *username_combo;
   GtkWidget *password_entry;
@@ -1230,7 +1180,8 @@ static void
 gis_account_page_dispose (GObject *object)
 {
   GisAccountPage *page = GIS_ACCOUNT_PAGE (object);
-  GisAccountPagePrivate *priv = page->priv;
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (page);
+
   if (priv->realmd_watch)
     g_bus_unwatch_name (priv->realmd_watch);
 
@@ -1249,7 +1200,9 @@ gis_account_page_dispose (GObject *object)
 static GtkWidget *
 gis_account_page_get_action_widget (GisPage *page)
 {
-  return GIS_ACCOUNT_PAGE (page)->priv->action;
+  GisAccountPage *account = GIS_ACCOUNT_PAGE (page);
+  GisAccountPagePrivate *priv = gis_account_page_get_instance_private (account);
+  return priv->action;
 }
 
 static void
@@ -1268,17 +1221,15 @@ gis_account_page_class_init (GisAccountPageClass *klass)
   page_class->locale_changed = gis_account_page_locale_changed;
   page_class->get_action_widget = gis_account_page_get_action_widget;
   page_class->apply = gis_account_page_apply;
+  page_class->save_data = gis_account_page_save_data;
   object_class->constructed = gis_account_page_constructed;
   object_class->dispose = gis_account_page_dispose;
-
-  g_type_class_add_private (object_class, sizeof(GisAccountPagePrivate));
 }
 
 static void
 gis_account_page_init (GisAccountPage *page)
 {
   g_resources_register (account_get_resource ());
-  page->priv = GET_PRIVATE (page);
 }
 
 void
