@@ -41,57 +41,78 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
-G_DEFINE_TYPE (GisGoaPage, gis_goa_page, GIS_TYPE_PAGE);
-
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GIS_TYPE_GOA_PAGE, GisGoaPagePrivate))
-
 struct _GisGoaPagePrivate {
   GoaClient *goa_client;
+  GtkWidget *dialog;
   gboolean accounts_exist;
 };
+typedef struct _GisGoaPagePrivate GisGoaPagePrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GisGoaPage, gis_goa_page, GIS_TYPE_PAGE);
 
 #define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE(page)->builder,(name)))
 #define WID(name) OBJ(GtkWidget*,name)
 
+static void
+on_have_providers (GObject       *source,
+                   GAsyncResult  *res,
+                   gpointer       user_data)
+{
+  GList *providers;
+  GList *l;
+  GError *error = NULL;
+  GisGoaPage *page = GIS_GOA_PAGE (user_data);
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
+
+  if (!goa_provider_get_all_finish (&providers, res, &error))
+    goto out;
+  
+  for (l = providers; l != NULL; l = l->next)
+    {
+      GoaProvider *provider;
+
+      provider = GOA_PROVIDER (l->data);
+      goa_panel_add_account_dialog_add_provider (GOA_PANEL_ADD_ACCOUNT_DIALOG (priv->dialog), provider);
+    }
+
+  g_list_foreach (providers, (GFunc) g_object_unref, NULL);
+  g_list_free (providers);
+
+ out:
+  if (error)
+    {
+      g_printerr ("Failed to list providers: %s\n", error->message);
+      g_error_free (error);
+    }
+}
+
 static void
 show_online_account_dialog (GtkButton *button,
                             gpointer   user_data)
 {
   GisGoaPage *page = GIS_GOA_PAGE (user_data);
-  GisGoaPagePrivate *priv = page->priv;
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
   GtkWindow *parent;
   GtkWidget *dialog;
   gint response;
-  GList *providers;
-  GList *l;
   GoaObject *object;
   GError *error;
 
-  providers = NULL;
-
   parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (page)));
 
-  dialog = goa_panel_add_account_dialog_new (priv->goa_client);
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+  priv->dialog = goa_panel_add_account_dialog_new (priv->goa_client);
+  gtk_window_set_transient_for (GTK_WINDOW (priv->dialog), parent);
 
-  providers = goa_provider_get_all ();
-  for (l = providers; l != NULL; l = l->next)
-    {
-      GoaProvider *provider;
+  goa_provider_get_all (on_have_providers, page);
 
-      provider = GOA_PROVIDER (l->data);
-      goa_panel_add_account_dialog_add_provider (GOA_PANEL_ADD_ACCOUNT_DIALOG (dialog), provider);
-    }
-
-  gtk_widget_show_all (dialog);
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_show_all (priv->dialog);
+  response = gtk_dialog_run (GTK_DIALOG (priv->dialog));
   if (response != GTK_RESPONSE_OK)
     {
       gtk_widget_destroy (dialog);
       goto out;
     }
 
-  error = NULL;
   object = goa_panel_add_account_dialog_get_account (GOA_PANEL_ADD_ACCOUNT_DIALOG (dialog), &error);
 
   if (object == NULL)
@@ -113,10 +134,8 @@ show_online_account_dialog (GtkButton *button,
         }
       g_error_free (error);
     }
-
  out:
-  g_list_foreach (providers, (GFunc) g_object_unref, NULL);
-  g_list_free (providers);
+  return;
 }
 
 static void
@@ -181,7 +200,7 @@ confirm_remove_account (GtkButton *button, gpointer user_data)
 static void
 update_visibility (GisGoaPage *page)
 {
-  GisGoaPagePrivate *priv = page->priv;
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
   GList *accounts;
 
   accounts = goa_client_get_accounts (priv->goa_client);
@@ -284,7 +303,7 @@ remove_account_from_list (GisGoaPage *page, GoaObject *object)
 static void
 populate_account_list (GisGoaPage *page)
 {
-  GisGoaPagePrivate *priv = page->priv;
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
   GList *accounts, *l;
   GoaObject *object;
 
@@ -327,7 +346,7 @@ network_status_changed (GNetworkMonitor *monitor,
                         gpointer         user_data)
 {
   GisGoaPage *page = GIS_GOA_PAGE (user_data);
-  GisGoaPagePrivate *priv = page->priv;
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
   GisAssistant *assistant = gis_driver_get_assistant (GIS_PAGE (page)->driver);
 
   /* Ignore the network change if we're the current page or if an account
@@ -362,7 +381,7 @@ static void
 gis_goa_page_constructed (GObject *object)
 {
   GisGoaPage *page = GIS_GOA_PAGE (object);
-  GisGoaPagePrivate *priv = page->priv;
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
   GtkWidget *button;
   GError *error = NULL;
   GNetworkMonitor *network_monitor = g_network_monitor_get_default ();
@@ -413,7 +432,7 @@ static void
 gis_goa_page_dispose (GObject *object)
 {
   GisGoaPage *page = GIS_GOA_PAGE (object);
-  GisGoaPagePrivate *priv = page->priv;
+  GisGoaPagePrivate *priv = gis_goa_page_get_instance_private (page);
   GNetworkMonitor *network_monitor = g_network_monitor_get_default ();
 
   g_clear_object (&priv->goa_client);
@@ -439,15 +458,12 @@ gis_goa_page_class_init (GisGoaPageClass *klass)
   page_class->locale_changed = gis_goa_page_locale_changed;
   object_class->constructed = gis_goa_page_constructed;
   object_class->dispose = gis_goa_page_dispose;
-  
-  g_type_class_add_private (object_class, sizeof(GisGoaPagePrivate));
 }
 
 static void
 gis_goa_page_init (GisGoaPage *page)
 {
   g_resources_register (goa_get_resource ());
-  page->priv = GET_PRIVATE (page);
 }
 
 void
