@@ -38,19 +38,19 @@
 #include <locale.h>
 #include <gtk/gtk.h>
 
-G_DEFINE_TYPE (GisLanguagePage, gis_language_page, GIS_TYPE_PAGE);
-
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GIS_TYPE_LANGUAGE_PAGE, GisLanguagePagePrivate))
-
 struct _GisLanguagePagePrivate
 {
   GtkWidget *language_chooser;
   GDBusProxy *localed;
   GPermission *permission;
   const gchar *new_locale_id;
+  guint selection_done_source;
 
   GCancellable *cancellable;
 };
+typedef struct _GisLanguagePagePrivate GisLanguagePagePrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GisLanguagePage, gis_language_page, GIS_TYPE_PAGE);
 
 #define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE (page)->builder,(name)))
 #define WID(name) OBJ(GtkWidget*,name)
@@ -58,7 +58,7 @@ struct _GisLanguagePagePrivate
 static void
 set_localed_locale (GisLanguagePage *self)
 {
-  GisLanguagePagePrivate *priv = self->priv;
+  GisLanguagePagePrivate *priv = gis_language_page_get_instance_private (self);
   GVariantBuilder *b;
   gchar *s;
 
@@ -80,12 +80,11 @@ change_locale_permission_acquired (GObject      *source,
                                    GAsyncResult *res,
                                    gpointer      data)
 {
-  GisLanguagePagePrivate *priv;
+  GisLanguagePage *page = GIS_LANGUAGE_PAGE (data);
+  GisLanguagePagePrivate *priv = gis_language_page_get_instance_private (page);
   GError *error = NULL;
   gboolean allowed;
 
-  priv = GIS_LANGUAGE_PAGE (data)->priv;
-
   allowed = g_permission_acquire_finish (priv->permission, res, &error);
   if (error) {
       if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -95,7 +94,7 @@ change_locale_permission_acquired (GObject      *source,
   }
 
   if (allowed)
-    set_localed_locale (GIS_LANGUAGE_PAGE (data));
+    set_localed_locale (page);
 }
 
 static void
@@ -110,14 +109,34 @@ user_loaded (GObject    *object,
   g_free (new_locale_id);
 }
 
-static void
-set_language (GisLanguagePage *page)
+static gboolean
+_selection_done (gpointer driver)
 {
-  GisLanguagePagePrivate *priv = page->priv;
-  GisDriver *driver = GIS_PAGE (page)->driver;
-  ActUser *user;
+  gis_assistant_next_page (gis_driver_get_assistant (GIS_DRIVER (driver)));
 
-  priv->new_locale_id = cc_language_chooser_get_language (CC_LANGUAGE_CHOOSER (priv->language_chooser));
+  return FALSE;
+}
+
+static void
+language_changed (CcLanguageChooser  *chooser,
+                  GParamSpec         *pspec,
+                  GisLanguagePage    *page)
+{
+  GisLanguagePagePrivate *priv = gis_language_page_get_instance_private (page);
+  ActUser *user;
+  GisDriver *driver;
+
+  if (priv->selection_done_source > 0)
+    {
+      g_source_remove (priv->selection_done_source);
+      priv->selection_done_source = 0;
+    }
+
+  priv->new_locale_id = cc_language_chooser_get_language (chooser);
+  driver = GIS_PAGE (page)->driver;
+
+  setlocale (LC_MESSAGES, priv->new_locale_id);
+  gis_driver_locale_changed (driver);
 
   if (gis_driver_get_mode (driver) == GIS_DRIVER_MODE_NEW_USER) {
       if (g_permission_get_allowed (priv->permission)) {
@@ -141,19 +160,9 @@ set_language (GisLanguagePage *page)
                       g_strdup (priv->new_locale_id));
 
   gis_driver_set_user_language (driver, priv->new_locale_id);
-}
 
-static void
-language_changed (CcLanguageChooser  *chooser,
-                  GParamSpec         *pspec,
-                  GisLanguagePage    *page)
-{
-  GisLanguagePagePrivate *priv = page->priv;
-  GisDriver *driver = GIS_PAGE (page)->driver;
-
-  set_language (page);
-  setlocale (LC_MESSAGES, priv->new_locale_id);
-  gis_driver_locale_changed (driver);
+  priv->selection_done_source = g_timeout_add (500, _selection_done,
+                                               (gpointer)driver);
 }
 
 static void
@@ -162,7 +171,7 @@ localed_proxy_ready (GObject      *source,
                      gpointer      data)
 {
   GisLanguagePage *self = data;
-  GisLanguagePagePrivate *priv;
+  GisLanguagePagePrivate *priv = gis_language_page_get_instance_private (self);
   GDBusProxy *proxy;
   GError *error = NULL;
 
@@ -175,15 +184,14 @@ localed_proxy_ready (GObject      *source,
       return;
   }
 
-  priv = self->priv;
   priv->localed = proxy;
 }
 
-static void
+ static void
 gis_language_page_constructed (GObject *object)
 {
   GisLanguagePage *page = GIS_LANGUAGE_PAGE (object);
-  GisLanguagePagePrivate *priv = page->priv;
+  GisLanguagePagePrivate *priv = gis_language_page_get_instance_private (page);
   GDBusConnection *bus;
 
   g_type_ensure (CC_TYPE_LANGUAGE_CHOOSER);
@@ -215,8 +223,6 @@ gis_language_page_constructed (GObject *object)
       g_object_unref (bus);
   }
 
-  set_language (page);
-
   gis_page_set_complete (GIS_PAGE (page), TRUE);
   gtk_widget_show (GTK_WIDGET (page));
 }
@@ -231,10 +237,12 @@ static void
 gis_language_page_dispose (GObject *object)
 {
   GisLanguagePage *page = GIS_LANGUAGE_PAGE (object);
-  GisLanguagePagePrivate *priv = page->priv;
+  GisLanguagePagePrivate *priv = gis_language_page_get_instance_private (page);
 
+  g_source_remove (priv->selection_done_source);
   g_clear_object (&priv->permission);
   g_clear_object (&priv->localed);
+  g_clear_object (&priv->cancellable);
 }
 
 static void
@@ -247,15 +255,12 @@ gis_language_page_class_init (GisLanguagePageClass *klass)
   page_class->locale_changed = gis_language_page_locale_changed;
   object_class->constructed = gis_language_page_constructed;
   object_class->dispose = gis_language_page_dispose;
-
-  g_type_class_add_private (object_class, sizeof(GisLanguagePagePrivate));
 }
 
 static void
 gis_language_page_init (GisLanguagePage *page)
 {
   g_resources_register (language_get_resource ());
-  page->priv = GET_PRIVATE (page);
 }
 
 void
